@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-plugins-engine.c 6264 2008-05-05 11:00:38Z sfre $
+ * $Id: gedit-plugins-engine.c 6498 2008-09-07 13:50:57Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -101,7 +101,7 @@ gedit_plugins_engine_load_dir (GeditPluginsEngine *engine,
 	d = g_dir_open (dir, 0, &error);
 	if (!d)
 	{
-		g_warning (error->message);
+		g_warning ("%s", error->message);
 		g_error_free (error);
 		return;
 	}
@@ -151,7 +151,7 @@ gedit_plugins_engine_load_dir (GeditPluginsEngine *engine,
 static void
 gedit_plugins_engine_load_all (GeditPluginsEngine *engine)
 {
-	GSList *active_plugins;
+	GSList *active_plugins = NULL;
 	const gchar *home;
 	const gchar *pdirs_env;
 	gchar **pdirs;
@@ -194,6 +194,8 @@ gedit_plugins_engine_load_all (GeditPluginsEngine *engine)
 		gedit_plugins_engine_load_dir (engine, pdirs[i], active_plugins);
 
 	g_strfreev (pdirs);
+	g_slist_foreach (active_plugins, (GFunc) g_free, NULL);
+	g_slist_free (active_plugins);
 }
 
 static void
@@ -342,7 +344,6 @@ gedit_plugins_engine_get_plugin_info (GeditPluginsEngine *engine,
 static gboolean
 load_plugin_module (GeditPluginInfo *info)
 {
-	gchar *path;
 	gchar *dirname;
 
 	gedit_debug (DEBUG_PLUGINS);
@@ -375,6 +376,8 @@ load_plugin_module (GeditPluginInfo *info)
 						   "module-name", info->module_name,
 						   NULL));
 
+	g_free (dirname);
+
 	if (!g_type_module_use (G_TYPE_MODULE (info->module)))
 	{
 		g_warning ("Cannot load plugin '%s' since file '%s' cannot be read.",
