@@ -25,12 +25,13 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-window.h 5064 2006-08-07 16:42:45Z pborelli $
+ * $Id: gedit-window.h 6275 2008-05-10 09:38:26Z pborelli $
  */
 
 #ifndef __GEDIT_WINDOW_H__
 #define __GEDIT_WINDOW_H__
 
+#include <gio/gio.h>
 #include <gtk/gtk.h>
 
 #include <gedit/gedit-tab.h>
@@ -53,7 +54,6 @@ typedef enum
  */
 #define GEDIT_TYPE_WINDOW              (gedit_window_get_type())
 #define GEDIT_WINDOW(obj)              (G_TYPE_CHECK_INSTANCE_CAST((obj), GEDIT_TYPE_WINDOW, GeditWindow))
-#define GEDIT_WINDOW_CONST(obj)        (G_TYPE_CHECK_INSTANCE_CAST((obj), GEDIT_TYPE_WINDOW, GeditWindow const))
 #define GEDIT_WINDOW_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST((klass), GEDIT_TYPE_WINDOW, GeditWindowClass))
 #define GEDIT_IS_WINDOW(obj)           (G_TYPE_CHECK_INSTANCE_TYPE((obj), GEDIT_TYPE_WINDOW))
 #define GEDIT_IS_WINDOW_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), GEDIT_TYPE_WINDOW))
@@ -149,7 +149,10 @@ GtkWidget	*gedit_window_get_statusbar		(GeditWindow         *window);
 GtkUIManager	*gedit_window_get_ui_manager		(GeditWindow         *window);
 
 GeditWindowState gedit_window_get_state 		(GeditWindow         *window);
-	
+
+GeditTab        *gedit_window_get_tab_from_location	(GeditWindow         *window,
+							 GFile               *location);
+
 GeditTab        *gedit_window_get_tab_from_uri		(GeditWindow         *window,
 							 const gchar         *uri);
 /*
@@ -164,6 +167,9 @@ gboolean	 _gedit_window_is_removing_tabs		(GeditWindow         *window);
 G_CONST_RETURN gchar
 		*_gedit_window_get_default_path 	(GeditWindow         *window);
 
+void		 _gedit_window_set_default_path 	(GeditWindow         *window,
+							 const gchar         *uri);
+
 void		 _gedit_window_set_saving_session_state	(GeditWindow         *window,
 							 gboolean             saving_session);
 
