@@ -188,10 +188,12 @@ static void next_files_async 				    (GFileEnumerator * enumerator,
 
 GEDIT_PLUGIN_DEFINE_TYPE_WITH_CODE (GeditFileBrowserStore, gedit_file_browser_store,
 			G_TYPE_OBJECT,
-			G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_MODEL,
-					       gedit_file_browser_store_iface_init)
-			G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_DRAG_SOURCE,
-					       gedit_file_browser_store_drag_source_init))
+			GEDIT_PLUGIN_IMPLEMENT_INTERFACE (gedit_file_browser_store_tree_model,
+							  GTK_TYPE_TREE_MODEL,
+							  gedit_file_browser_store_iface_init)
+			GEDIT_PLUGIN_IMPLEMENT_INTERFACE (gedit_file_browser_store_drag_source,
+							  GTK_TYPE_TREE_DRAG_SOURCE,
+							  gedit_file_browser_store_drag_source_init))
 
 /* Properties */
 enum {
@@ -1523,6 +1525,9 @@ file_browser_node_unload (GeditFileBrowserStore * model,
 			  FileBrowserNode * node, gboolean remove_children)
 {
 	FileBrowserNodeDir *dir;
+	
+	if (node == NULL)
+		return;
 
 	if (!NODE_IS_DIR (node) || !NODE_LOADED (node))
 		return;
@@ -2704,12 +2709,12 @@ gedit_file_browser_store_set_virtual_root (GeditFileBrowserStore * model,
 GeditFileBrowserStoreResult
 gedit_file_browser_store_set_virtual_root_from_string
     (GeditFileBrowserStore * model, gchar const *root) {
-	GFile * file = g_file_new_for_uri (root);
-	gchar *str, *str1;
+	GFile *file;
 
 	g_return_val_if_fail (GEDIT_IS_FILE_BROWSER_STORE (model),
 			      GEDIT_FILE_BROWSER_STORE_RESULT_NO_CHANGE);
 
+	file = g_file_new_for_uri (root);
 	if (file == NULL) {
 		g_warning ("Invalid uri (%s)", root);
 		return GEDIT_FILE_BROWSER_STORE_RESULT_NO_CHANGE;
@@ -2733,8 +2738,10 @@ gedit_file_browser_store_set_virtual_root_from_string
 	}
 
 	if (!g_file_has_prefix (file, model->priv->root->file)) {
-		str = gedit_file_browser_utils_file_display (model->priv->root->file);
-		str1 = gedit_file_browser_utils_file_display (file);
+		gchar *str, *str1;
+
+		str = g_file_get_parse_name (model->priv->root->file);
+		str1 = g_file_get_parse_name (file);
 
 		g_warning
 		    ("Virtual root (%s) is not below actual root (%s)",
@@ -3024,7 +3031,7 @@ gedit_file_browser_store_refresh (GeditFileBrowserStore * model)
 {
 	g_return_if_fail (GEDIT_IS_FILE_BROWSER_STORE (model));
 
-	if (model->priv->root == NULL)
+	if (model->priv->root == NULL || model->priv->virtual_root == NULL)
 		return;
 
 	/* Clear the model */
