@@ -47,7 +47,10 @@ struct _GeditAutomaticSpellChecker {
 	GeditDocument		*doc;
 	GSList 			*views;
 	
-	GtkTextMark 		*mark_insert;
+	GtkTextMark 		*mark_insert_start;
+	GtkTextMark		*mark_insert_end;
+	gboolean 		 deferred_check;
+
 	GtkTextTag 		*tag_highlight;
 	GtkTextMark		*mark_click;
 
@@ -77,7 +80,7 @@ check_word (GeditAutomaticSpellChecker *spell, GtkTextIter *start, GtkTextIter *
 						gtk_text_iter_get_offset (end));
 	*/
 
-	if (!gedit_spell_checker_check_word (spell->spell_checker, word, -1, NULL)) 
+	if (!gedit_spell_checker_check_word (spell->spell_checker, word, -1)) 
 	{
 		/*
 		g_print ("Apply tag: [%d - %d]\n", gtk_text_iter_get_offset (start),
@@ -93,13 +96,20 @@ check_word (GeditAutomaticSpellChecker *spell, GtkTextIter *start, GtkTextIter *
 }
 
 static void
-check_range (GeditAutomaticSpellChecker *spell, GtkTextIter start, GtkTextIter end) 
+check_range (GeditAutomaticSpellChecker *spell, 
+	     GtkTextIter                 start, 
+	     GtkTextIter                 end,
+	     gboolean                    force_all) 
 {
 	/* we need to "split" on word boundaries.
 	 * luckily, Pango knows what "words" are 
 	 * so we don't have to figure it out. */
 
-	GtkTextIter wstart, wend;
+	GtkTextIter wstart;
+	GtkTextIter wend;
+	GtkTextIter cursor; 
+	GtkTextIter precursor;
+  	gboolean    highlight;
 
 	/*
 	g_print ("Check range: [%d - %d]\n", gtk_text_iter_get_offset (&start),
@@ -116,7 +126,8 @@ check_range (GeditAutomaticSpellChecker *spell, GtkTextIter start, GtkTextIter e
 		{
 			gtk_text_iter_backward_word_start (&start);
 		} 
-		else {
+		else 
+		{
 			/* if we're neither at the beginning nor inside a word,
 			 * me must be in some spaces.
 			 * skip forward to the beginning of the next word. */
@@ -125,6 +136,16 @@ check_range (GeditAutomaticSpellChecker *spell, GtkTextIter start, GtkTextIter e
 				gtk_text_iter_backward_word_start (&start);
 		}
 	}
+
+	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (spell->doc), 
+					  &cursor,
+					  gtk_text_buffer_get_insert (GTK_TEXT_BUFFER (spell->doc)));
+  	 
+	precursor = cursor;
+	gtk_text_iter_backward_char (&precursor);
+	
+  	highlight = gtk_text_iter_has_tag (&cursor, spell->tag_highlight) ||
+  	            gtk_text_iter_has_tag (&precursor, spell->tag_highlight);
 	
 	gtk_text_buffer_remove_tag (GTK_TEXT_BUFFER (spell->doc), 
 				    spell->tag_highlight, 
@@ -135,7 +156,7 @@ check_range (GeditAutomaticSpellChecker *spell, GtkTextIter start, GtkTextIter e
 	 * An iter at offset 0 seems to always be inside a word,
   	 * even if it's not.  Possibly a pango bug.
 	 */
-  	if (gtk_text_iter_get_offset(&start) == 0) 
+  	if (gtk_text_iter_get_offset (&start) == 0) 
 	{
 		gtk_text_iter_forward_word_end(&start);
 		gtk_text_iter_backward_word_start(&start);
@@ -145,12 +166,31 @@ check_range (GeditAutomaticSpellChecker *spell, GtkTextIter start, GtkTextIter e
 	
 	while (gtk_text_iter_compare (&wstart, &end) < 0) 
 	{
+		gboolean inword; 
+
 		/* move wend to the end of the current word. */
 		wend = wstart;
 		
 		gtk_text_iter_forward_word_end (&wend);
 
-		check_word (spell, &wstart, &wend);
+		inword = (gtk_text_iter_compare (&wstart, &cursor) < 0) &&
+			 (gtk_text_iter_compare (&cursor, &wend) <= 0);
+  	 
+		if (inword && !force_all)
+		{
+			/* this word is being actively edited,
+			 * only check if it's already highligted,
+			 * otherwise defer this check until later. */
+			if (highlight)
+				check_word (spell, &wstart, &wend);
+			else
+				spell->deferred_check = TRUE;
+		} 
+		else 
+		{
+			check_word (spell, &wstart, &wend);
+			spell->deferred_check = FALSE;
+		}
 
 		/* now move wend to the beginning of the next word, */
 		gtk_text_iter_forward_word_end (&wend);
@@ -166,6 +206,22 @@ check_range (GeditAutomaticSpellChecker *spell, GtkTextIter start, GtkTextIter e
 	}
 }
 
+static void
+check_deferred_range (GeditAutomaticSpellChecker *spell, 
+		      gboolean                    force_all) 
+{
+	GtkTextIter start, end;
+
+	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (spell->doc), 
+					  &start, 
+					  spell->mark_insert_start);
+	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (spell->doc),
+					  &end, 
+					  spell->mark_insert_end);
+
+	check_range (spell, start, end, force_all);
+}
+
 /* insertion works like this:
  *  - before the text is inserted, we mark the position in the buffer.
  *  - after the text is inserted, we see where our mark is and use that and
@@ -177,7 +233,7 @@ static void
 insert_text_before (GtkTextBuffer *buffer, GtkTextIter *iter,
 		gchar *text, gint len, GeditAutomaticSpellChecker *spell) 
 {
-	gtk_text_buffer_move_mark (buffer, spell->mark_insert, iter);
+	gtk_text_buffer_move_mark (buffer, spell->mark_insert_start, iter);
 }
 
 static void
@@ -187,9 +243,11 @@ insert_text_after (GtkTextBuffer *buffer, GtkTextIter *iter,
 	GtkTextIter start;
 
 	/* we need to check a range of text. */
-	gtk_text_buffer_get_iter_at_mark (buffer, &start, spell->mark_insert);
+	gtk_text_buffer_get_iter_at_mark (buffer, &start, spell->mark_insert_start);
 	
-	check_range (spell, start, *iter);
+	check_range (spell, start, *iter, FALSE);
+
+	gtk_text_buffer_move_mark (buffer, spell->mark_insert_end, iter);
 }
 
 /* deleting is more simple:  we're given the range of deleted text.
@@ -203,7 +261,18 @@ static void
 delete_range_after (GtkTextBuffer *buffer, GtkTextIter *start, GtkTextIter *end, 
 		GeditAutomaticSpellChecker *spell) 
 {
-	check_range (spell, *start, *end);
+	check_range (spell, *start, *end, FALSE);
+}
+
+static void
+mark_set (GtkTextBuffer              *buffer, 
+	  GtkTextIter                *iter,
+	  GtkTextMark                *mark, 
+	  GeditAutomaticSpellChecker *spell) 
+{
+	/* if the cursor has moved and there is a deferred check so handle it now */
+	if ((mark == gtk_text_buffer_get_insert (buffer)) && spell->deferred_check)
+		check_deferred_range (spell, FALSE);
 }
 
 static void
@@ -274,7 +343,7 @@ add_to_dictionary (GtkWidget *menuitem, GeditAutomaticSpellChecker *spell)
 					 &end, 
 					 FALSE);
 	
-	gedit_spell_checker_add_word_to_personal (spell->spell_checker, word, -1, NULL);
+	gedit_spell_checker_add_word_to_personal (spell->spell_checker, word, -1);
 
 	g_free (word);
 }
@@ -293,7 +362,7 @@ ignore_all (GtkWidget *menuitem, GeditAutomaticSpellChecker *spell)
 					 &end, 
 					 FALSE);
 	
-	gedit_spell_checker_add_word_to_session (spell->spell_checker, word, -1, NULL);
+	gedit_spell_checker_add_word_to_session (spell->spell_checker, word, -1);
 
 	g_free (word);
 }
@@ -313,17 +382,16 @@ replace_word (GtkWidget *menuitem, GeditAutomaticSpellChecker *spell)
 	newword =  g_object_get_qdata (G_OBJECT (menuitem), suggestion_id);
 	g_return_if_fail (newword != NULL);
 
-	gedit_document_begin_user_action (spell->doc);
+	gtk_text_buffer_begin_user_action (GTK_TEXT_BUFFER (spell->doc));
 
 	gtk_text_buffer_delete (GTK_TEXT_BUFFER (spell->doc), &start, &end);
 	gtk_text_buffer_insert (GTK_TEXT_BUFFER (spell->doc), &start, newword, -1);
 
-	gedit_document_end_user_action (spell->doc);
+	gtk_text_buffer_end_user_action (GTK_TEXT_BUFFER (spell->doc));
 
 	gedit_spell_checker_set_correction (spell->spell_checker, 
 				oldword, strlen (oldword),
-				newword, strlen (newword),
-				NULL);
+				newword, strlen (newword));
 
 	g_free (oldword);
 }
@@ -339,7 +407,7 @@ build_suggestion_menu (GeditAutomaticSpellChecker *spell, const gchar *word)
 	
 	topmenu = menu = gtk_menu_new();
 
-	suggestions = gedit_spell_checker_get_suggestions (spell->spell_checker, word, -1, NULL);
+	suggestions = gedit_spell_checker_get_suggestions (spell->spell_checker, word, -1);
 
 	list = suggestions;
 
@@ -398,11 +466,13 @@ build_suggestion_menu (GeditAutomaticSpellChecker *spell, const gchar *word)
 				 (GDestroyNotify)g_free);
 
 			g_free (label_text);
-			g_signal_connect (G_OBJECT (mi), "activate",
-					  G_CALLBACK (replace_word), spell);
-			
+			g_signal_connect (mi,
+					  "activate",
+					  G_CALLBACK (replace_word),
+					  spell);
+
 			count++;
-			
+
 			suggestions = g_slist_next (suggestions);
 		}
 	}
@@ -429,11 +499,13 @@ build_suggestion_menu (GeditAutomaticSpellChecker *spell, const gchar *word)
 				       gtk_image_new_from_stock (GTK_STOCK_GOTO_BOTTOM, 
 					       			 GTK_ICON_SIZE_MENU));
 	
-	g_signal_connect (G_OBJECT(mi), "activate",
-			  G_CALLBACK(ignore_all), spell);
-	
+	g_signal_connect (mi,
+			  "activate",
+			  G_CALLBACK(ignore_all),
+			  spell);
+
 	gtk_widget_show_all (mi);
-	
+
 	gtk_menu_shell_append (GTK_MENU_SHELL (topmenu), mi);
 
 	/* + Add to Dictionary */
@@ -442,8 +514,10 @@ build_suggestion_menu (GeditAutomaticSpellChecker *spell, const gchar *word)
 				       gtk_image_new_from_stock (GTK_STOCK_ADD, 
 					       			 GTK_ICON_SIZE_MENU));
 
-	g_signal_connect (G_OBJECT(mi), "activate",
-			  G_CALLBACK(add_to_dictionary), spell);
+	g_signal_connect (mi,
+			  "activate",
+			  G_CALLBACK (add_to_dictionary),
+			  spell);
 
 	gtk_widget_show_all (mi);
 	
@@ -495,7 +569,7 @@ gedit_automatic_spell_checker_recheck_all (GeditAutomaticSpellChecker *spell)
 
 	gtk_text_buffer_get_bounds (GTK_TEXT_BUFFER (spell->doc), &start, &end);
 
-	check_range (spell, start, end);
+	check_range (spell, start, end, TRUE);
 }
 
 static void 
@@ -517,9 +591,9 @@ add_word_signal_cb (GeditSpellChecker          *checker,
 }
 
 static void 
-set_language_cb (GeditSpellChecker          *checker,
-		 const GeditLanguage        *lang,
-		 GeditAutomaticSpellChecker *spell)
+set_language_cb (GeditSpellChecker               *checker,
+		 const GeditSpellCheckerLanguage *lang,
+		 GeditAutomaticSpellChecker      *spell)
 {
 	gedit_automatic_spell_checker_recheck_all (spell);
 }
@@ -547,6 +621,10 @@ button_press_event (GtkTextView *view,
 
 		GtkTextBuffer *buffer = gtk_text_view_get_buffer (view);
 
+		/* handle deferred check if it exists */
+  	        if (spell->deferred_check)
+			check_deferred_range (spell, TRUE);
+
 		gtk_text_view_window_to_buffer_coords (view, 
 				GTK_TEXT_WINDOW_TEXT, 
 				event->x, event->y,
@@ -572,6 +650,10 @@ popup_menu_event (GtkTextView *view, GeditAutomaticSpellChecker *spell)
 
 	buffer = gtk_text_view_get_buffer (view);
 
+	/* handle deferred check if it exists */
+	if (spell->deferred_check)
+		check_deferred_range (spell, TRUE);
+
 	gtk_text_buffer_get_iter_at_mark (buffer, &iter,
 					  gtk_text_buffer_get_insert (buffer));
 	gtk_text_buffer_move_mark (buffer, spell->mark_click, &iter);
@@ -580,19 +662,35 @@ popup_menu_event (GtkTextView *view, GeditAutomaticSpellChecker *spell)
 }
 
 static void
-tag_table_changed (GtkSourceTagTable          *table,
+tag_table_changed (GtkTextTagTable            *table,
 		   GeditAutomaticSpellChecker *spell)
 {
 	g_return_if_fail (spell->tag_highlight !=  NULL);
-	g_return_if_fail (GTK_TEXT_BUFFER (spell->doc)->tag_table != NULL);
-	g_return_if_fail (GTK_IS_SOURCE_TAG_TABLE (GTK_TEXT_BUFFER (spell->doc)->tag_table));
 
-	gtk_text_tag_set_priority (spell->tag_highlight, 
-				   gtk_text_tag_table_get_size (GTK_TEXT_BUFFER (spell->doc)->tag_table) - 1);
+	gtk_text_tag_set_priority (spell->tag_highlight,
+				   gtk_text_tag_table_get_size (table) - 1);
+}
+
+static void 
+tag_added_or_removed (GtkTextTagTable            *table,
+		      GtkTextTag                 *tag,
+		      GeditAutomaticSpellChecker *spell)
+{
+	tag_table_changed (table, spell);
+}
+
+static void 
+tag_changed (GtkTextTagTable            *table,
+	     GtkTextTag                 *tag,
+	     gboolean                    size_changed,
+	     GeditAutomaticSpellChecker *spell)
+{
+	tag_table_changed (table, spell);
 }
 
 GeditAutomaticSpellChecker *
-gedit_automatic_spell_checker_new (GeditDocument *doc, GeditSpellChecker *checker)
+gedit_automatic_spell_checker_new (GeditDocument     *doc,
+				   GeditSpellChecker *checker)
 {
 	GeditAutomaticSpellChecker *spell;
 
@@ -607,10 +705,8 @@ gedit_automatic_spell_checker_new (GeditDocument *doc, GeditSpellChecker *checke
 	spell = g_new0 (GeditAutomaticSpellChecker, 1);
 
 	spell->doc = doc;
+	spell->spell_checker = g_object_ref (checker);
 
-	g_object_ref (checker);
-	spell->spell_checker = checker;
-	
 	if (automatic_spell_checker_id == 0)
 		automatic_spell_checker_id = 
 			g_quark_from_static_string ("GeditAutomaticSpellCheckerID");
@@ -623,35 +719,36 @@ gedit_automatic_spell_checker_new (GeditDocument *doc, GeditSpellChecker *checke
 				 spell, 
 				 (GDestroyNotify)gedit_automatic_spell_checker_free_internal);
 
-	g_signal_connect (G_OBJECT (doc),
+	g_signal_connect (doc,
 			  "insert-text",
 			  G_CALLBACK (insert_text_before), 
 			  spell);
-	g_signal_connect_after (G_OBJECT (doc),
+	g_signal_connect_after (doc,
 			  "insert-text",
 			  G_CALLBACK (insert_text_after), 
 			  spell);
-	g_signal_connect_after (G_OBJECT (doc),
+	g_signal_connect_after (doc,
 			  "delete-range",
 			  G_CALLBACK (delete_range_after), 
 			  spell);
+	g_signal_connect (doc,
+			  "mark-set",
+			  G_CALLBACK (mark_set), 
+			  spell);
 
-	g_signal_connect (G_OBJECT (spell->spell_checker),
+	g_signal_connect (spell->spell_checker,
 			  "add_word_to_session",
 			  G_CALLBACK (add_word_signal_cb),
 			  spell);
-
-	g_signal_connect (G_OBJECT (spell->spell_checker),
+	g_signal_connect (spell->spell_checker,
 			  "add_word_to_personal",
 			  G_CALLBACK (add_word_signal_cb),
 			  spell);
-
-	g_signal_connect (G_OBJECT (spell->spell_checker),
+	g_signal_connect (spell->spell_checker,
 			  "clear_session",
 			  G_CALLBACK (clear_session_cb),
 			  spell);
-
-	g_signal_connect (G_OBJECT (spell->spell_checker),
+	g_signal_connect (spell->spell_checker,
 			  "set_language",
 			  G_CALLBACK (set_language_cb),
 			  spell);
@@ -663,34 +760,56 @@ gedit_automatic_spell_checker_new (GeditDocument *doc, GeditSpellChecker *checke
 				NULL);
 
 	g_return_val_if_fail (GTK_TEXT_BUFFER (doc)->tag_table != NULL, NULL);
-	g_return_val_if_fail (GTK_IS_SOURCE_TAG_TABLE (GTK_TEXT_BUFFER (doc)->tag_table), NULL);
 
 	gtk_text_tag_set_priority (spell->tag_highlight, 
 				   gtk_text_tag_table_get_size (GTK_TEXT_BUFFER (doc)->tag_table) - 1);
 
-	g_signal_connect (G_OBJECT (GTK_TEXT_BUFFER (doc)->tag_table),
-			  "changed",
-                          G_CALLBACK (tag_table_changed),
+	g_signal_connect (GTK_TEXT_BUFFER (doc)->tag_table,
+			  "tag-added",
+			  G_CALLBACK (tag_added_or_removed),
+			  spell);
+	g_signal_connect (GTK_TEXT_BUFFER (doc)->tag_table,
+			  "tag-removed",
+			  G_CALLBACK (tag_added_or_removed),
+			  spell);
+	g_signal_connect (GTK_TEXT_BUFFER (doc)->tag_table,
+			  "tag-changed",
+			  G_CALLBACK (tag_changed),
 			  spell);
 
 	/* we create the mark here, but we don't use it until text is
 	 * inserted, so we don't really care where iter points.  */
 	gtk_text_buffer_get_bounds (GTK_TEXT_BUFFER (doc), &start, &end);
 	
-	spell->mark_insert = gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
-					"gedit-automatic-spell-checker-insert");
+	spell->mark_insert_start = gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
+					"gedit-automatic-spell-checker-insert-start");
 
-	if (spell->mark_insert == NULL)
-		spell->mark_insert = 
+	if (spell->mark_insert_start == NULL)
+		spell->mark_insert_start = 
 			gtk_text_buffer_create_mark (GTK_TEXT_BUFFER (doc),
-						     "gedit-automatic-spell-checker-insert",
+						     "gedit-automatic-spell-checker-insert-start",
 						     &start, 
 						     TRUE);
 	else
 		gtk_text_buffer_move_mark (GTK_TEXT_BUFFER (doc),
-					   spell->mark_insert,
+					   spell->mark_insert_start,
 					   &start);
 
+	spell->mark_insert_end = gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
+					"gedit-automatic-spell-checker-insert-end");
+
+	if (spell->mark_insert_end == NULL)
+		spell->mark_insert_end = 
+			gtk_text_buffer_create_mark (GTK_TEXT_BUFFER (doc),
+						     "gedit-automatic-spell-checker-insert-end",
+						     &start, 
+						     TRUE);
+	else
+		gtk_text_buffer_move_mark (GTK_TEXT_BUFFER (doc),
+					   spell->mark_insert_end,
+					   &start);
+
+
 	spell->mark_click = gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
 					"gedit-automatic-spell-checker-click");
 
@@ -705,6 +824,7 @@ gedit_automatic_spell_checker_new (GeditDocument *doc, GeditSpellChecker *checke
 					   spell->mark_click,
 					   &start);
 
+	spell->deferred_check = FALSE;
 
 	return spell;
 }
@@ -726,6 +846,9 @@ gedit_automatic_spell_checker_free (GeditAutomaticSpellChecker *spell)
 	g_return_if_fail (spell != NULL);
 	g_return_if_fail (gedit_automatic_spell_checker_get_from_document (spell->doc) == spell);
 	
+	if (automatic_spell_checker_id == 0)
+		return;
+
 	g_object_set_qdata (G_OBJECT (spell->doc), automatic_spell_checker_id, NULL);	
 }
 
@@ -749,6 +872,12 @@ gedit_automatic_spell_checker_free_internal (GeditAutomaticSpellChecker *spell)
 					    spell->tag_highlight, 
 					    &start, 
 					    &end);
+
+		g_signal_handlers_disconnect_matched (G_OBJECT (table),
+					G_SIGNAL_MATCH_DATA,
+					0, 0, NULL, NULL,
+					spell);
+
 		gtk_text_tag_table_remove (table, spell->tag_highlight);
 	}
 		
@@ -774,6 +903,11 @@ gedit_automatic_spell_checker_free_internal (GeditAutomaticSpellChecker *spell)
 				0, 0, NULL, NULL,
 				spell);
 
+		g_signal_handlers_disconnect_matched (G_OBJECT (view),
+			G_SIGNAL_MATCH_DATA,
+			0, 0, NULL, NULL,
+			spell);
+
 		list = g_slist_next (list);
 	}
 
@@ -789,24 +923,23 @@ gedit_automatic_spell_checker_attach_view (
 {
 	g_return_if_fail (spell != NULL);
 	g_return_if_fail (GEDIT_IS_VIEW (view));
-	g_return_if_fail (gedit_view_get_document (view) == spell->doc);
 
-	g_signal_connect (G_OBJECT (gedit_view_get_gtk_text_view (view)), 
+	g_return_if_fail (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)) ==
+			  GTK_TEXT_BUFFER (spell->doc));
+
+	g_signal_connect (view,
 			  "button-press-event",
 			  G_CALLBACK (button_press_event), 
 			  spell);
-
-	g_signal_connect (G_OBJECT (gedit_view_get_gtk_text_view (view)), 
+	g_signal_connect (view,
 			  "popup-menu",
 			  G_CALLBACK (popup_menu_event), 
 			  spell);
-
-	g_signal_connect (G_OBJECT (view), 
+	g_signal_connect (view,
 			  "populate-popup",
 			  G_CALLBACK (populate_popup), 
 			  spell);
-	
-	g_signal_connect (G_OBJECT (view), 
+	g_signal_connect (view,
 			  "destroy",
 			  G_CALLBACK (view_destroy), 
 			  spell);
@@ -821,9 +954,16 @@ gedit_automatic_spell_checker_detach_view (
 {
 	g_return_if_fail (spell != NULL);
 	g_return_if_fail (GEDIT_IS_VIEW (view));
-	g_return_if_fail (gedit_view_get_document (view) == spell->doc);
+
+	g_return_if_fail (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)) ==
+			  GTK_TEXT_BUFFER (spell->doc));
 	g_return_if_fail (spell->views != NULL);
 
+	g_signal_handlers_disconnect_matched (G_OBJECT (view),
+			G_SIGNAL_MATCH_DATA,
+			0, 0, NULL, NULL,
+			spell);
+
 	g_signal_handlers_disconnect_matched (G_OBJECT (view),
 			G_SIGNAL_MATCH_DATA,
 			0, 0, NULL, NULL,
