@@ -32,11 +32,9 @@
 #endif
 
 #include <string.h>
-
-#include <glade/glade-xml.h>
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
-
+#include <gedit/gedit-utils.h>
 #include "gedit-spell-checker-dialog.h"
 #include "gedit-spell-checker-dialog-marshal.h"
 
@@ -46,9 +44,9 @@ struct _GeditSpellCheckerDialog
 
 	GeditSpellChecker 	*spell_checker;
 
-	gchar			*mispelled_word;
+	gchar			*misspelled_word;
 
-	GtkWidget 		*mispelled_word_label;
+	GtkWidget 		*misspelled_word_label;
 	GtkWidget		*word_entry;
 	GtkWidget		*check_word_button;
 	GtkWidget		*ignore_button;
@@ -219,60 +217,51 @@ gedit_spell_checker_dialog_class_init (GeditSpellCheckerDialogClass * klass)
 static void
 gedit_spell_checker_dialog_init (GeditSpellCheckerDialog *dlg)
 {
-	GladeXML *gui;
+	GtkWidget *error_widget;
 	GtkWidget *content;
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *cell;
 	GtkTreeSelection *selection;
-
+	gboolean ret;
+	gchar *root_objects[] = {
+		"content",
+		NULL
+	};
+	
 	g_return_if_fail (dlg != NULL);
-	
+
 	dlg->spell_checker = NULL;
-	dlg->mispelled_word = NULL;
+	dlg->misspelled_word = NULL;
 
-	gui = glade_xml_new (GEDIT_GLADEDIR "spell-checker.glade2",
-			     "content", NULL);
+	ret = gedit_utils_get_ui_objects (GEDIT_UIDIR "spell-checker.ui",
+		root_objects,
+		&error_widget,
 
-	if (!gui) 
+		"content", &content,
+		"misspelled_word_label", &dlg->misspelled_word_label,
+		"word_entry", &dlg->word_entry,
+		"check_word_button", &dlg->check_word_button,
+		"ignore_button", &dlg->ignore_button,
+		"ignore_all_button", &dlg->ignore_all_button,
+		"change_button", &dlg->change_button,
+		"change_all_button", &dlg->change_all_button,
+		"add_word_button", &dlg->add_word_button,
+		"close_button", &dlg->close_button,
+		"suggestions_list", &dlg->suggestions_list,
+		"language_label", &dlg->language_label,
+		NULL);
+
+	if (!ret)
 	{
-		g_warning ("Could not find spell-checker.glade2, reinstall gedit.\n");
+		gtk_widget_show (error_widget);
+			
+		gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (dlg)->vbox),
+					     error_widget);
+		
 		return;
 	}
 
-	content = glade_xml_get_widget (gui, "content");
-	
-	dlg->mispelled_word_label = glade_xml_get_widget (gui, "mispelled_word_label");
-	dlg->word_entry = glade_xml_get_widget (gui, "word_entry");
-	dlg->check_word_button = glade_xml_get_widget (gui, "check_word_button");
-	dlg->ignore_button = glade_xml_get_widget (gui, "ignore_button");
-	dlg->ignore_all_button = glade_xml_get_widget (gui, "ignore_all_button");
-	dlg->change_button = glade_xml_get_widget (gui, "change_button");
-	dlg->change_all_button = glade_xml_get_widget (gui, "change_all_button");
-	dlg->add_word_button = glade_xml_get_widget (gui, "add_word_button");
-	dlg->close_button = glade_xml_get_widget (gui, "close_button");
-	dlg->suggestions_list = glade_xml_get_widget (gui, "suggestions_list");
-	dlg->language_label = glade_xml_get_widget (gui, "language_label");
-	
-	if ((content == NULL) ||
-	    (dlg->mispelled_word_label == NULL) ||
-	    (dlg->word_entry == NULL) ||
-	    (dlg->check_word_button == NULL) ||
-	    (dlg->ignore_button == NULL) || 
-	    (dlg->ignore_all_button == NULL) ||
-	    (dlg->change_button == NULL) ||
-	    (dlg->change_all_button == NULL) ||
-	    (dlg->add_word_button == NULL) ||
-	    (dlg->close_button == NULL) ||
-	    (dlg->suggestions_list == NULL) ||
-	    (dlg->language_label == NULL))
-	{
-		g_object_unref (G_OBJECT (gui));
-
-		g_warning ("Could not find the required widgets inside spell-checker.glade2.\n");
-		return;
-	}
-
-	gtk_label_set_label (GTK_LABEL (dlg->mispelled_word_label), "");
+	gtk_label_set_label (GTK_LABEL (dlg->misspelled_word_label), "");
 	gtk_widget_set_sensitive (dlg->word_entry, FALSE);
 	gtk_widget_set_sensitive (dlg->check_word_button, FALSE);
 	gtk_widget_set_sensitive (dlg->ignore_button, FALSE);
@@ -284,6 +273,7 @@ gedit_spell_checker_dialog_init (GeditSpellCheckerDialog *dlg)
 	gtk_label_set_label (GTK_LABEL (dlg->language_label), "");
 			
 	gtk_container_add (GTK_CONTAINER (dlg), content);
+	g_object_unref (content);
 
 	gtk_window_set_resizable (GTK_WINDOW (dlg), FALSE);
 	gtk_window_set_title (GTK_WINDOW (dlg), _("Check Spelling"));
@@ -303,12 +293,11 @@ gedit_spell_checker_dialog_init (GeditSpellCheckerDialog *dlg)
 	gtk_tree_view_append_column (GTK_TREE_VIEW (dlg->suggestions_list), column);
 
 	gtk_tree_view_set_search_column (GTK_TREE_VIEW (dlg->suggestions_list),
-			COLUMN_SUGGESTIONS);
+					 COLUMN_SUGGESTIONS);
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (dlg->suggestions_list));
 
-	gtk_tree_selection_set_mode (selection,
-				   GTK_SELECTION_SINGLE);
+	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
 
 	/* Set default button */
 	GTK_WIDGET_SET_FLAGS (dlg->change_button, GTK_CAN_DEFAULT);
@@ -317,31 +306,27 @@ gedit_spell_checker_dialog_init (GeditSpellCheckerDialog *dlg)
 	gtk_entry_set_activates_default (GTK_ENTRY (dlg->word_entry), TRUE);
 
 	/* Connect signals */
-	g_signal_connect (G_OBJECT (dlg->word_entry), "changed",
+	g_signal_connect (dlg->word_entry, "changed",
 			  G_CALLBACK (word_entry_changed_handler), dlg);
-	g_signal_connect (G_OBJECT (dlg->close_button), "clicked",
+	g_signal_connect (dlg->close_button, "clicked",
 			  G_CALLBACK (close_button_clicked_handler), dlg);
 	g_signal_connect (selection, "changed", 
 			  G_CALLBACK (suggestions_list_selection_changed_handler), 
 			  dlg);
-	g_signal_connect (G_OBJECT (dlg->check_word_button), "clicked",
+	g_signal_connect (dlg->check_word_button, "clicked",
 			  G_CALLBACK (check_word_button_clicked_handler), dlg);
-	g_signal_connect (G_OBJECT (dlg->add_word_button), "clicked",
+	g_signal_connect (dlg->add_word_button, "clicked",
 			  G_CALLBACK (add_word_button_clicked_handler), dlg);
-	
-	g_signal_connect (G_OBJECT (dlg->ignore_button), "clicked",
+	g_signal_connect (dlg->ignore_button, "clicked",
 			  G_CALLBACK (ignore_button_clicked_handler), dlg);
-	g_signal_connect (G_OBJECT (dlg->ignore_all_button), "clicked",
+	g_signal_connect (dlg->ignore_all_button, "clicked",
 			  G_CALLBACK (ignore_all_button_clicked_handler), dlg);
-	g_signal_connect (G_OBJECT (dlg->change_button), "clicked",
+	g_signal_connect (dlg->change_button, "clicked",
 			  G_CALLBACK (change_button_clicked_handler), dlg);
-	g_signal_connect (G_OBJECT (dlg->change_all_button), "clicked",
+	g_signal_connect (dlg->change_all_button, "clicked",
 			  G_CALLBACK (change_all_button_clicked_handler), dlg);
-
-	g_signal_connect (G_OBJECT (dlg->suggestions_list), "row-activated",
+	g_signal_connect (dlg->suggestions_list, "row-activated",
 			  G_CALLBACK (suggestions_list_row_activated_handler), dlg);
-
-	g_object_unref (G_OBJECT (gui));
 }
 
 static void
@@ -355,10 +340,10 @@ gedit_spell_checker_dialog_destroy (GtkObject *object)
 		dlg->spell_checker = NULL;
 	}
 
-	if (dlg->mispelled_word != NULL)
+	if (dlg->misspelled_word != NULL)
 	{
-		g_free (dlg->mispelled_word);
-		dlg->mispelled_word = NULL;
+		g_free (dlg->misspelled_word);
+		dlg->misspelled_word = NULL;
 	}
 
 	GTK_OBJECT_CLASS (parent_class)->destroy (object);
@@ -404,8 +389,8 @@ gedit_spell_checker_dialog_new_from_spell_checker (GeditSpellChecker *spell)
 void
 gedit_spell_checker_dialog_set_spell_checker (GeditSpellCheckerDialog *dlg, GeditSpellChecker *spell)
 {
-	const GeditLanguage* language;
-	gchar *lang;
+	const GeditSpellCheckerLanguage* language;
+	const gchar *lang;
 	gchar *tmp;
 	
 	g_return_if_fail (GEDIT_IS_SPELL_CHECKER_DIALOG (dlg));
@@ -419,15 +404,14 @@ gedit_spell_checker_dialog_set_spell_checker (GeditSpellCheckerDialog *dlg, Gedi
 	
 	language = gedit_spell_checker_get_language (dlg->spell_checker);
 
-	lang = gedit_language_to_string (language);
+	lang = gedit_spell_checker_language_to_string (language);
 	tmp = g_strdup_printf("<b>%s</b>", lang);
-	g_free (lang);
 
 	gtk_label_set_label (GTK_LABEL (dlg->language_label), tmp);
 	g_free (tmp);
 
-	if (dlg->mispelled_word != NULL)
-		gedit_spell_checker_dialog_set_mispelled_word (dlg, dlg->mispelled_word, -1);
+	if (dlg->misspelled_word != NULL)
+		gedit_spell_checker_dialog_set_misspelled_word (dlg, dlg->misspelled_word, -1);
 	else
 		gtk_list_store_clear (GTK_LIST_STORE (dlg->suggestions_list_model));
 
@@ -435,54 +419,38 @@ gedit_spell_checker_dialog_set_spell_checker (GeditSpellCheckerDialog *dlg, Gedi
 }
 
 void
-gedit_spell_checker_dialog_set_mispelled_word (GeditSpellCheckerDialog *dlg, 
-		const gchar* word, gint len)
+gedit_spell_checker_dialog_set_misspelled_word (GeditSpellCheckerDialog *dlg, 
+						const gchar             *word,
+						gint                     len)
 {
 	gchar *tmp;
-	GSList *sug, *list;
-	GError *error = NULL;
+	GSList *sug;
 	
 	g_return_if_fail (GEDIT_IS_SPELL_CHECKER_DIALOG (dlg));
 	g_return_if_fail (word != NULL);
 
 	g_return_if_fail (dlg->spell_checker != NULL);
-	g_return_if_fail (!gedit_spell_checker_check_word (dlg->spell_checker, word, -1, NULL));
+	g_return_if_fail (!gedit_spell_checker_check_word (dlg->spell_checker, word, -1));
 
 	/* build_suggestions_list */
-	if (dlg->mispelled_word != NULL)
-		g_free (dlg->mispelled_word);
+	if (dlg->misspelled_word != NULL)
+		g_free (dlg->misspelled_word);
 
-	dlg->mispelled_word = g_strdup (word);
+	dlg->misspelled_word = g_strdup (word);
 	
 	tmp = g_strdup_printf("<b>%s</b>", word);
-	gtk_label_set_label (GTK_LABEL (dlg->mispelled_word_label), tmp);
+	gtk_label_set_label (GTK_LABEL (dlg->misspelled_word_label), tmp);
 	g_free (tmp);
 
 	sug = gedit_spell_checker_get_suggestions (dlg->spell_checker,
-		       dlg->mispelled_word, -1, &error);
+		       				   dlg->misspelled_word, 
+		       				   -1);
 	
-	if (error != NULL)
-	{
-		g_warning ("%s", error->message);
-		g_error_free (error);
-
-		gtk_list_store_clear (GTK_LIST_STORE (dlg->suggestions_list_model));
-	}
-	else
-		update_suggestions_list_model (dlg, sug);
+	update_suggestions_list_model (dlg, sug);
 
 	/* free the suggestion list */
-	list = sug;
-	
-	while (sug)
-	{
-		g_free (sug->data);
-
-		sug = g_slist_next (sug);
-	}
-
-
-	g_slist_free (list);
+	g_slist_foreach (sug, (GFunc)g_free, NULL);
+	g_slist_free (sug);
 
 	gtk_widget_set_sensitive (dlg->ignore_button, TRUE);
 	gtk_widget_set_sensitive (dlg->ignore_all_button, TRUE);
@@ -596,13 +564,15 @@ static void
 check_word_button_clicked_handler (GtkButton *button, GeditSpellCheckerDialog *dlg)
 {
 	const gchar *word;
+	gssize len;
 	
 	g_return_if_fail (GEDIT_IS_SPELL_CHECKER_DIALOG (dlg));
 
 	word = gtk_entry_get_text (GTK_ENTRY (dlg->word_entry));
-	g_return_if_fail (g_utf8_strlen (word, -1) > 0);
+	len = strlen (word);
+	g_return_if_fail (len > 0);
 	
-	if (gedit_spell_checker_check_word (dlg->spell_checker, word, -1, NULL))
+	if (gedit_spell_checker_check_word (dlg->spell_checker, word, len))
 	{
 		GtkListStore *store;
 		GtkTreeIter iter;
@@ -619,33 +589,17 @@ check_word_button_clicked_handler (GtkButton *button, GeditSpellCheckerDialog *d
 	}
 	else
 	{
-		GSList *sug, *list;
-		GError *error = NULL;
+		GSList *sug;
 
 		sug = gedit_spell_checker_get_suggestions (dlg->spell_checker,
-				word, -1, &error);
+							   word,
+							   len);
 	
-		if (error != NULL)
-		{
-			g_warning ("%s", error->message);
-			g_error_free (error);
-
-			gtk_list_store_clear (GTK_LIST_STORE (dlg->suggestions_list_model));
-		}
-		else
-			update_suggestions_list_model (dlg, sug);
+		update_suggestions_list_model (dlg, sug);
 
 		/* free the suggestion list */
-		list = sug;
-		
-		while (sug)
-		{
-			g_free (sug->data);
-
-			sug = g_slist_next (sug);
-		}
-
-		g_slist_free (list);
+		g_slist_foreach (sug, (GFunc)g_free, NULL);
+		g_slist_free (sug);
 	}
 }
 
@@ -653,21 +607,15 @@ static void
 add_word_button_clicked_handler (GtkButton *button, GeditSpellCheckerDialog *dlg)
 {
 	gchar *word;	
-	GError *error = NULL;
 	
 	g_return_if_fail (GEDIT_IS_SPELL_CHECKER_DIALOG (dlg));
-	g_return_if_fail (dlg->mispelled_word != NULL);
+	g_return_if_fail (dlg->misspelled_word != NULL);
 
 	gedit_spell_checker_add_word_to_personal (dlg->spell_checker, 
-			dlg->mispelled_word, -1, &error);
+						  dlg->misspelled_word, 
+						  -1);
 
-	if (error)
-	{
-		g_warning ("%s", error->message);
-		g_error_free (error);
-	}
-
-	word = g_strdup (dlg->mispelled_word);
+	word = g_strdup (dlg->misspelled_word);
 
 	g_signal_emit (G_OBJECT (dlg), signals [ADD_WORD_TO_PERSONAL], 0, word);
 
@@ -680,9 +628,9 @@ ignore_button_clicked_handler (GtkButton *button, GeditSpellCheckerDialog *dlg)
 	gchar *word;
 	
 	g_return_if_fail (GEDIT_IS_SPELL_CHECKER_DIALOG (dlg));
-	g_return_if_fail (dlg->mispelled_word != NULL);
+	g_return_if_fail (dlg->misspelled_word != NULL);
 
-	word = g_strdup (dlg->mispelled_word);
+	word = g_strdup (dlg->misspelled_word);
 
 	g_signal_emit (G_OBJECT (dlg), signals [IGNORE], 0, word);
 
@@ -693,21 +641,15 @@ static void
 ignore_all_button_clicked_handler (GtkButton *button, GeditSpellCheckerDialog *dlg)
 {
 	gchar *word;
-	GError *error = NULL;
 	
 	g_return_if_fail (GEDIT_IS_SPELL_CHECKER_DIALOG (dlg));
-	g_return_if_fail (dlg->mispelled_word != NULL);
+	g_return_if_fail (dlg->misspelled_word != NULL);
 
 	gedit_spell_checker_add_word_to_session (dlg->spell_checker,
-			dlg->mispelled_word, -1, &error);
+						 dlg->misspelled_word, 
+						 -1);
 
-	if (error)
-	{
-		g_warning ("%s", error->message);
-		g_error_free (error);
-	}
-
-	word = g_strdup (dlg->mispelled_word);
+	word = g_strdup (dlg->misspelled_word);
 
 	g_signal_emit (G_OBJECT (dlg), signals [IGNORE_ALL], 0, word);
 
@@ -719,25 +661,19 @@ change_button_clicked_handler (GtkButton *button, GeditSpellCheckerDialog *dlg)
 {
 	gchar *word;
 	gchar *change;
-	GError *error = NULL;
-		
+
 	g_return_if_fail (GEDIT_IS_SPELL_CHECKER_DIALOG (dlg));
-	g_return_if_fail (dlg->mispelled_word != NULL);
+	g_return_if_fail (dlg->misspelled_word != NULL);
 
 	change = g_strdup (gtk_entry_get_text (GTK_ENTRY (dlg->word_entry)));
 	g_return_if_fail (change != NULL);
-	g_return_if_fail (strlen (change) > 0);
+	g_return_if_fail (*change != '\0');
 
 	gedit_spell_checker_set_correction (dlg->spell_checker, 
-			dlg->mispelled_word, -1, change, -1, &error);
+					    dlg->misspelled_word, -1, 
+					    change, -1);
 	
-	if (error)
-	{
-		g_warning ("%s", error->message);
-		g_error_free (error);
-	}
-	
-	word = g_strdup (dlg->mispelled_word);
+	word = g_strdup (dlg->misspelled_word);
 
 	g_signal_emit (G_OBJECT (dlg), signals [CHANGE], 0, word, change);
 
@@ -762,27 +698,19 @@ change_all_button_clicked_handler (GtkButton *button, GeditSpellCheckerDialog *d
 {
 	gchar *word;
 	gchar *change;
-	
-	GError *error = NULL;
-	
 		
 	g_return_if_fail (GEDIT_IS_SPELL_CHECKER_DIALOG (dlg));
-	g_return_if_fail (dlg->mispelled_word != NULL);
+	g_return_if_fail (dlg->misspelled_word != NULL);
 
 	change = g_strdup (gtk_entry_get_text (GTK_ENTRY (dlg->word_entry)));
 	g_return_if_fail (change != NULL);
-	g_return_if_fail (strlen (change) > 0);
+	g_return_if_fail (*change != '\0');
 
 	gedit_spell_checker_set_correction (dlg->spell_checker, 
-			dlg->mispelled_word, -1, change, -1, &error);
+					    dlg->misspelled_word, -1,
+					    change, -1);
 	
-	if (error)
-	{
-		g_warning ("%s", error->message);
-		g_error_free (error);
-	}
-	
-	word = g_strdup (dlg->mispelled_word);
+	word = g_strdup (dlg->misspelled_word);
 
 	g_signal_emit (G_OBJECT (dlg), signals [CHANGE_ALL], 0, word, change);
 
@@ -798,7 +726,7 @@ gedit_spell_checker_dialog_set_completed (GeditSpellCheckerDialog *dlg)
 	g_return_if_fail (GEDIT_IS_SPELL_CHECKER_DIALOG (dlg));
 
 	tmp = g_strdup_printf("<b>%s</b>", _("Completed spell checking"));
-	gtk_label_set_label (GTK_LABEL (dlg->mispelled_word_label), 
+	gtk_label_set_label (GTK_LABEL (dlg->misspelled_word_label), 
 			     tmp);
 	g_free (tmp);
 
